#pragma fragment
#ifdef GL_ES
precision highp float;
#endif
#pragma common

varying vec2 v_texCoord;
uniform float u_time;

#pragma vertex

attribute vec2 a_position;
uniform   mat4 u_model;
uniform   mat4 u_view;
uniform   mat4 u_projection;

uniform vec2 u_size;

void main()
{
    gl_Position = u_projection * u_view * u_model * vec4(a_position * u_size, 0.0, 1.0);
    v_texCoord = (a_position + 1.0) * 0.5 * u_size;
}

#pragma fragment

uniform sampler2D u_texture;
uniform float u_opacity;

// Properties of the tiled atlas quad. Contains texture coordinates of its top left corner (x, y) and its size (z, w).
uniform vec4 u_quadTopLeftTexCoord;
uniform vec4 u_noiseQuadTopLeftTexCoord;

uniform vec4 u_scale;
uniform vec4 u_speed;
uniform float u_opacityMultiplier;

float sum( vec4 v ) { return v.x+v.y+v.z; }

void main()
{
    vec2 cloudScale = vec2(u_scale.x);
    vec2 cloudScale2 = vec2(u_scale.y);
    vec2 noiseUvScale = vec2(mix(u_scale.z, u_scale.w, sin(u_time * 0.005 * u_speed.x) * 0.5 + 0.5)); // vec2(5.0 / 15000.0); //

    vec2 noiseCoord = u_noiseQuadTopLeftTexCoord.xy + fract(v_texCoord * noiseUvScale + vec2(0.008, 0.004) * u_time * u_speed.y) * u_noiseQuadTopLeftTexCoord.zw;

    float k = texture2D(u_texture, noiseCoord).x;

    vec2 coord1 = u_quadTopLeftTexCoord.xy + fract(v_texCoord * cloudScale + vec2(0.005, 0.008) * u_time * u_speed.z) * u_quadTopLeftTexCoord.zw;
    vec2 coord2 = u_quadTopLeftTexCoord.xy + fract(v_texCoord * cloudScale2 + vec2(-0.003, 0.005) * u_time * u_speed.w) * u_quadTopLeftTexCoord.zw;

    vec4 c1 = texture2D(u_texture, coord1);
    vec4 c2 = texture2D(u_texture, coord2);

    gl_FragColor = mix(c1, c2, k);
    gl_FragColor.a *= u_opacity * u_opacityMultiplier;
}
