#extension GL_EXT_shader_framebuffer_fetch : enable
#extension GL_ARM_shader_framebuffer_fetch : enable

#pragma fragment
#ifdef GL_ES
precision mediump float;
#endif
#pragma common
varying vec2 v_texCoord;
#pragma vertex

attribute vec2 a_position;
attribute vec2 a_uv;
uniform   mat4 u_model;
uniform   mat4 u_view;
uniform   mat4 u_projection;

void main()
{
    gl_Position = u_projection * u_view * u_model * vec4(a_position, 0.0, 1.0);
    v_texCoord = vec2(a_uv.x, 1.0 - a_uv.y);
}

#pragma fragment

uniform sampler2D u_texture;
uniform vec4 u_color;

uniform vec4 u_fade_bounds;
uniform vec3 u_dodge_color;
uniform vec2 u_dodge_opacity;
uniform vec2 u_brightness_contrast;
uniform vec4 u_additiveColor;

float applyBrightnessContrast(float c) {
    float r = min(c + u_brightness_contrast.x, 1.0);
    r = (r - 0.5) * max(u_brightness_contrast.y + 1.0, 0.0) + 0.5;
    return r;
}
vec3 applyBrightnessContrast(vec3 c) {
    return vec3(applyBrightnessContrast(c.r), applyBrightnessContrast(c.g), applyBrightnessContrast(c.b));
}

float blendColorDodge(float base, float blend) {
	return (blend==1.0)?blend:min(base/(1.0-blend),1.0);
}
vec3 blendColorDodge(vec3 base, vec3 blend) {
	return vec3(blendColorDodge(base.r,blend.r),blendColorDodge(base.g,blend.g),blendColorDodge(base.b,blend.b));
}

void main()
{
    vec3 mask = texture2D(u_texture, v_texCoord).rgb;
    float lightStrength = max(mask.r, mask.b);

    #if (defined(GL_EXT_shader_framebuffer_fetch) || defined(GL_ARM_shader_framebuffer_fetch)) && !defined(BE_LOW_QUALITY_SHADERS)

    #ifdef GL_EXT_shader_framebuffer_fetch
    vec3 originalColor = gl_LastFragData[0].rgb * gl_LastFragData[0].a;
    #elif GL_ARM_shader_framebuffer_fetch
    vec3 originalColor = gl_LastFragColorARM.rgb * gl_LastFragColorARM.a;
    #endif

    vec3 dodgedColor = applyBrightnessContrast(
        blendColorDodge(originalColor, u_dodge_color * mix(u_dodge_opacity.x, u_dodge_opacity.y, mask.g))
    );

    vec3 originalDarkenColor = mix(originalColor, u_additiveColor.rgb, u_additiveColor.a);
    vec3 c = mix(originalDarkenColor, dodgedColor, lightStrength);
    gl_FragColor = vec4(c, 1.0);

    #else
    vec4 darknessAdditiveColor = vec4(0.0, 0.0, 0.1, 0.3);
    vec4 lightAdditiveColor = vec4(1.0, 0.945, 0.7, 0.6);

    gl_FragColor = mix(darknessAdditiveColor, lightAdditiveColor, lightStrength);
    #endif

}
