#pragma fragment
#ifdef GL_ES
precision highp float;
#endif
#pragma common

varying vec2 v_pos;
varying vec4 v_dirAngleRadiusIsCircle;

#pragma vertex

attribute vec2 a_positionLocal;
attribute vec2 a_positionWorld;
attribute vec4 a_dirAngleRadiusIsCircle;

uniform   mat4 u_view;
uniform   mat4 u_projection;

void main()
{
    gl_Position = u_projection * u_view * vec4(a_positionWorld, 0.0, 1.0);
    v_pos = a_positionLocal;
    v_dirAngleRadiusIsCircle = a_dirAngleRadiusIsCircle;
}

#pragma fragment

uniform vec4 u_fade_bounds;

void main()
{
    float direction = v_dirAngleRadiusIsCircle.x;
    float halfFovAngle = v_dirAngleRadiusIsCircle.y * 0.5;
    float visibleRadius = v_dirAngleRadiusIsCircle.z;
    float isCircle = v_dirAngleRadiusIsCircle.w;

    vec2 diff = v_pos / visibleRadius;
    float d = dot(diff, diff);
    d = sqrt(d); // mix(sqrt(d), d, u_fadeWeight);

    float angle = atan(v_pos.y, v_pos.x) - direction;
    if (angle < -3.14) angle += 2.0 * 3.14; // todo: optimize, remove 'if'
    else if (angle > 3.14) angle -= 2.0 * 3.14;
    float absAngle = abs(angle);

    float fadeOutEdge = smoothstep(halfFovAngle * u_fade_bounds.x, halfFovAngle * u_fade_bounds.y, absAngle);
    float dodgeFadeEdge = smoothstep(halfFovAngle * u_fade_bounds.z, halfFovAngle * u_fade_bounds.w, absAngle);

    float base = 1.0 - d;

    vec4 circleColor = vec4(0.0, 0.0, base, 1.0);
    vec4 sectorColor = vec4(base * fadeOutEdge, dodgeFadeEdge, 0.0, 1.0);
    gl_FragColor = mix(sectorColor, circleColor, isCircle);
}
