#pragma fragment
#ifdef GL_ES
precision mediump float;
#endif
#pragma common

/*
uv.x: [-angle, angle]
uv.y: 0 - bottomR, 1 - R, x - outerR
*/
varying vec2 v_uv;

#pragma vertex

attribute vec2 a_position;
attribute vec2 a_uv;
uniform   mat4 u_model;
uniform   mat4 u_view;
uniform   mat4 u_projection;

void main()
{
    gl_Position = u_projection * u_view * u_model * vec4(a_position, 0.0, 1.0);
    v_uv = a_uv;
}

#pragma fragment

uniform float u_time;
uniform sampler2D u_texture;
// Properties of the tiled atlas quad. Contains texture coordinates of its top left corner (x, y) and its size (z, w).
uniform vec4 u_noiseQuadTopLeftTexCoord;
uniform float u_fovAngle;

uniform vec4 u_noise_params;
uniform vec2 u_width_params;

void main()
{
    vec2 noiseCoord = u_noiseQuadTopLeftTexCoord.xy + fract(vec2(v_uv.x * u_noise_params.x + u_time * u_noise_params.y * 0.1, u_time * u_noise_params.z * 0.1)) * u_noiseQuadTopLeftTexCoord.zw;
    float noise = texture2D(u_texture, noiseCoord).x;
    vec2 uv = v_uv;
    uv.y *= 1.0 + (noise * 2.0 - 1.0) * u_noise_params.w;

    float horizontalFade = 1.0 - smoothstep(u_fovAngle - 0.035, u_fovAngle, abs(uv.x));

    float yOuter = max(uv.y - 1.0, 0.0);
    float fadeOuter = smoothstep(0.0, yOuter, u_width_params.x * 0.1);

    float yStep = step(uv.y, 1.0);

    float inner = min(uv.y + noise * 0.3, 1.0);
    float outer = mix(fadeOuter, inner * inner, yStep);

    float core = smoothstep(0.0, abs(uv.y - 1.0), u_width_params.y);

    gl_FragColor = vec4(core * (sin(u_time * 4.0) * 0.1 + 0.9), 1.0, 1.0, outer * horizontalFade * horizontalFade);
}
