#pragma fragment
#ifdef GL_ES
precision highp float;
#endif
#pragma common

varying vec2 v_pos;

#pragma vertex

attribute vec2 a_position;
uniform   mat4 u_model;
uniform   mat4 u_view;
uniform   mat4 u_projection;

void main()
{
    gl_Position = u_projection * u_view * u_model * vec4(a_position, 0.0, 1.0);
    v_pos = a_position;
}

#pragma fragment

uniform float u_direction;
uniform float u_visibleRadius;

uniform float u_fovAngle;
uniform vec4 u_fade_bounds;
uniform float u_time;

void main()
{
    vec2 diff = v_pos / u_visibleRadius;
    float d = dot(diff, diff);

    float angle = atan(v_pos.y, v_pos.x) - u_direction;
    if (angle < -3.14) angle += 2.0 * 3.14; // todo: optimize, remove 'if'
    else if (angle > 3.14) angle -= 2.0 * 3.14;
    float absAngle = abs(angle);

    float horizontalRayDist = 100.0 / u_visibleRadius;
    float horD = mod(d - u_time * 0.25, horizontalRayDist) / horizontalRayDist;
    // 1.0 - smoothstep(0.0, 0.03, abs(horD - 0.5));

    float fadeOutEdge = smoothstep(u_fovAngle * u_fade_bounds.x, u_fovAngle * u_fade_bounds.y, absAngle);

    float rays = horD * horD * horD;
    vec3 c = vec3(rays * 0.14, 0.14 + rays * 0.42, rays * 0.14);
    c *= (1.0 - d) * fadeOutEdge;
    gl_FragColor = vec4(c, 1.0);
}
